--------------------------------------------------------------------------------
--- Globals
--------------------------------------------------------------------------------
Global("onReaction", { })
Global("onEvent", { })

--------------------------------------------------------------------------------
--- Common Functions
--------------------------------------------------------------------------------
function RegisterReactionHandlers(reactions)
  for k, v in reactions do
  common.RegisterReactionHandler( v, k ) 
  end
end

function RegisterEventHandlers(events)
  for k, v in events do
    common.RegisterEventHandler(v, k)
  end
end

function EqualPlacementPlain(placementPlain1, placementPlain2)
  if not (placementPlain1 or placementPlain2) then
    return false
  end
  if (placementPlain1.alignX ~= placementPlain2.alignX) or
     (placementPlain1.alignY ~= placementPlain2.alignY) or
     (placementPlain1.posX ~= placementPlain2.posX) or
     (placementPlain1.posY ~= placementPlain2.posY) or
     (placementPlain1.highPosX ~= placementPlain2.highPosX) or
     (placementPlain1.highPosY ~= placementPlain2.highPosY) or
     (placementPlain1.sizeX ~= placementPlain2.sizeX) or
     (placementPlain1.sizeY ~= placementPlain2.sizeY) then
    return false
  end
  return true
end

function GetTableSize( t )
	if not t then return 0 end
	return t[0] and table.getn(t)+1 or table.getn(t)
end

function ShowTable(t)
  for k, v in t do
    LogInfo("t[",k,"] = ",v)
  end
end

function CallWhenCharacterAvailable( func )
  if avatar and avatar.IsExist() then
    func()
  else
    common.RegisterEventHandler(func, "EVENT_AVATAR_CREATED")
  end
end

--- Displays all fields and values ​​as well as the function of the object
function researchObj (tab, obj)

	tab = tab .. "  "

	--- Restrict recursion
	if string.len (tab)> 50 then
		LogInfo ("recursion is limited!")
		return
	end

	local metaTable = getmetatable (obj)
	if metaTable then
		---- Show the function of the object
		for k, v in pairs (metaTable) do
			LogInfo (tab, k ,":=", v)
		end
	end
	if type (obj) == "table" then
		---- Show the fields (variables) of the table
		if GetTableSize (obj) == 0
			then LogInfo (tab, "{}")
			return
		end
		for k, v in pairs (obj) do
			LogInfo (tab, k ,":=", v, "{", type (v), "}")
			if type (v) == "table" and k ~= "__index" and k ~= "_G" then
				--- "__index" - He was exactly the same table is a nested loop that leads to
				researchObj (tab, v)
			end
		end
	else
		LogInfo (tab, "_:=", obj, "{", type (obj), "}")
	end
end 

function PlacementPlainFromRealRect(realRect)
  return { 
           alignX = WIDGET_ALIGN_LOW, 
		   alignY = WIDGET_ALIGN_LOW, 
		   posX = realRect.x1, 
		   posY = realRect.y1,
		   highPosX = 0,
		   highPosY = 0,
		   sizeX = realRect.x2 - realRect.x1,
		   sizeY = realRect.y2 - realRect.y1,
		   sizingX = 0,
		   sizingY = 0
		}
end

function CopyTable(t)
  local result = { }
  for k, v in t do
    result[k] = v
  end
  return result
end

function string.split(str, delim, maxNb)
    -- Eliminate bad cases...
    if string.find(str, delim) == nil then
        return { self }
    end
    if maxNb == nil or maxNb < 1 then
        maxNb = 0    -- No limit
    end
    local result = {}
    local pat = "(.-)" .. delim .. "()"
    local nb = 0
    local lastPos
    for part, pos in string.gfind(str, pat) do
        nb = nb + 1
        result[nb] = part
        lastPos = pos
        if nb == maxNb then break end
    end
    -- Handle the last field
    if nb ~= maxNb then
        result[nb + 1] = string.sub(str, lastPos)
    end
    return result
end

--------------------------------------------------------------------------------
--- Timer
--------------------------------------------------------------------------------
function StartTimer( wtTimerTemplate, interval, tickFunction)
  local wtTimer = mainForm:CreateWidgetByDesc(wtTimerTemplate:GetWidgetDesc())
  common.RegisterEventHandler(function(params)
    if params.effectType == ET_FADE and params.wtOwner:IsEqual( wtTimer ) then 
	  wtTimer:PlayFadeEffect(0.0, 1.0, interval, EA_MONOTONOUS_INCREASE)
	  tickFunction()
	end
  end, "EVENT_EFFECT_FINISHED")
  wtTimer:PlayFadeEffect(0.0, 1.0, interval, EA_MONOTONOUS_INCREASE)
end