--------------------------------------------------------------------------------
--- Locals
--------------------------------------------------------------------------------
local wtPanel = View.Point
local wtControl3D = nil
local damagevisualization = nil
local targetpanel = mainForm:CreateWidgetByDesc(wtPanel.widget:GetWidgetDesc())
targetpanel:SetBackgroundColor({ r = 1.0, g = 1.0, b = 1.0, a = 0.0 })
local placementplain = targetpanel:GetPlacementPlain()
placementplain.alignX = WIDGET_ALIGN_CENTER
placementplain.alignY = WIDGET_ALIGN_CENTER
placementplain.sizeX = TargetWidgetWidth
placementplain.sizeY = TargetWidgetHeight
targetpanel:SetPlacementPlain(placementplain)

local lastTarget = nil
local targetPanelLayout = nil

-- These table is needed. When I place an own widget to a panel of an other addon,
-- then the EVENT_EFFECT_FINISHED will not work for the own widget.
-- As key i use the InstanceIdNumber of the 3D-Panel Item and the value is the 
-- own widget that is on the Targetspanel
local targetWidgets = { }
--------------------------------------------------------------------------------
--- Functions
--------------------------------------------------------------------------------
function CreateTargetPanelLayout()
  local result = { }
  result.grid = { }
  result.widgets = { }
  function result:CanPlace(x, y, xend, yend)
    for yy = y, yend do
	  for xx = x, xend do 	    
	    if self.grid[yy*TargetGridColumns+xx+1] then
		  return false
		end
	  end
	end
	return true
  end
  function result:Register(widget, x, y, sizeX, sizeY)
    local instance = common.RequestIntegerByInstanceId(widget:GetInstanceId())
	self.widgets[instance] = widget
    local widthInColumns, heightInRows = math.ceil(sizeX / TargetGridCellWidth), math.ceil(sizeY / TargetGridCellHeight)
	for yy = y, y+heightInRows-1 do
	  for xx = x, x+widthInColumns-1 do
	    self.grid[yy*TargetGridColumns+xx+1] = instance
	  end
	end
  end
  function result:Lock(x, y, columns, rows)
    for yy = y, y+rows-1 do
	  for xx = x, x+columns-1 do
	    self.grid[yy*TargetGridColumns+xx+1] = -1
	  end
	end
  end
  function result:Unregister(widget)
    local instance = common.RequestIntegerByInstanceId(widget:GetInstanceId())
    widget = self.widgets[instance]
    if widget then
      for k, v in self.grid do
	    if v == instance then	    
		  self.grid[k] = nil		
	    end
	  end
	end
  end
  function result:CalculatePosition(sizeX, sizeY)
	local widthInColumns, heightInRows = math.ceil(sizeX / TargetGridCellWidth), math.ceil(sizeY / TargetGridCellHeight)

	-- rx: The Columns where the text will placed in
	-- ry: The Row where the text will placed in
	-- rd: The Distance of the MiddlePoint from the TextView to the MiddlePoint of the Panel
	local rx, ry, rd, rleft, rtop = nil, nil, 1/0, nil, nil

	-- This part will look for the perfect place for the widget
	for y = 0, TargetGridRows-heightInRows do
	  for x = 0, TargetGridColumns-widthInColumns do
	    --LogInfo(x, ",", y, " => ", x+widthInColumns-1, ",",y+heightInRows-1)
	    if self:CanPlace(x, y, x+widthInColumns-1, y+heightInRows-1) then
		  local left, top
		  if x < HalfTargetGridColumns then
		    left = (x + widthInColumns) * TargetGridCellWidth - sizeX
		  else
		    left = x * TargetGridCellWidth
		  end
		  if x < HalfTargetGridRows then
		    top = (y + heightInRows) * TargetGridCellHeight - sizeY
		  else
		    top = y * TargetGridCellHeight
		  end
		  local widgetMiddlePointX = left + sizeX / 2
		  local widgetMiddlePointY = top + sizeY / 2

		  local a = widgetMiddlePointX - HalfTargetWidgetWidth
		  local b = widgetMiddlePointY - HalfTargetWidgetHeight
		  local distance = math.sqrt(a * a + b * b)

		  --LogInfo("Distance: ", distance, " < ", rd," X: ", x, "Y: ", y, " Left: ", left, " Top: ", top)
		  if distance < rd then
		    rd = distance
			rx = x
			ry = y
			rleft = left
			rtop = top
		  end
		end
	  end
	end

	if rx then
	  return rx, ry, rleft, rtop
	end
	return nil
  end
  return result
end
--------------------------------------------------------------------------------
--- EVENTS
--------------------------------------------------------------------------------
onEvent["EVENT_EFFECT_FINISHED"] = function(params)
  local element = params.wtOwner

  if params.effectType == ET_FADE then
    -- Text
    if GetTextByKey(element) then
	  FreeText(element)	  
	
	  local elementInstanceId = common.RequestIntegerByInstanceId(element:GetInstanceId())
	  local targetElement = targetWidgets[elementInstanceId]
	  if targetElement and GetTextByKey(targetElement) then
	    FreeText(targetElement)	
	    targetPanelLayout:Unregister(targetElement)
	    targetWidgets[elementInstanceId] = nil
	  end
	end
	-- Texture
    if GetTextureByKey(element) then
      FreeTexture(element)

	  local elementInstanceId = common.RequestIntegerByInstanceId(element:GetInstanceId())
	  local targetElement = targetWidgets[elementInstanceId]
	   if targetElement and GetTextureByKey(targetElement) then
	    FreeText(targetElement)	
	    targetPanelLayout:Unregister(targetElement)
	    targetWidgets[elementInstanceId] = nil
	  end
	end
  end
end
onEvent["EVENT_AVATAR_PRIMARY_TARGET_CHANGED"] = function(params)
  if lastTarget and object.IsExist(lastTarget) then
    local panel = Get3DWidget(lastTarget)
	panel:Show(true)  
  end
  lastTarget = avatar.GetTarget()
  if lastTarget then
    local panel = Get3DWidget(lastTarget)
	targetpanel:Show(true)
	local children = targetpanel:GetNamedChildren()
    for i = 0, GetTableSize( children ) - 1 do
	  children[i]:Show(false)
	  targetPanelLayout:Unregister(children[i])
	end
	panel:Show(false)
  else
    targetpanel:Show(false)
  end
end
onEvent["EVENT_HEALING_RECEIVED"] = function(params)
    local avatarId = avatar.GetId()
	local petId = unit.GetActivePet(avatarId)
    if (params.healerId == avatarId or (petId and params.healerId == petId)
	   or avatar.GetTarget() == params.unitId) and (params.unitId ~= avatar.GetId()) then  
	  damagevisualization:Show(true)
	  local panel = Get3DWidget(params.unitId)
	  local targetIsAvatarTarget = params.unitId == avatar.GetTarget()
	  panel:Show(not targetIsAvatarTarget)
	  local textWidget, texturePanel = 
		CreateVisual(panel, GetRandomPosX(_G["3DWidgetWidth"]), GetRandomPosY(_G["3DWidgetHeight"]),
					 params.heal, GetTextFormatHeal(params.isCritical),
					 GetTexture(params), _G["3DWidgetScale"])

	  if targetIsAvatarTarget then
	    textWidget:Show(false)
		texturePanel:Show(false)
	    local textScale = GetTextScale(params.amount)
		local sizeX, sizeY = CalculateSize(textScale, TargetWidgetScale)
		local x, y, left, top = targetPanelLayout:CalculatePosition(sizeX, sizeY)
		if x then
	      local targetTextWidget, targetTexturePanel = 
		    CreateVisual(targetpanel, left, top,
			  		     params.heal, GetTextFormatHeal(params.isCritical),
					     GetTexture(params), _G["TargetWidgetScale"])
						
		  targetPanelLayout:Register(targetTextWidget, x, y, sizeX, sizeY)
		  targetWidgets[common.RequestIntegerByInstanceId(textWidget:GetInstanceId())] = targetTextWidget
		  targetWidgets[common.RequestIntegerByInstanceId(texturePanel:GetInstanceId())] = targetTexturePanel
		end
	  end
	end
  end
onEvent["EVENT_UNIT_DAMAGE_RECEIVED"] = function(params)
    local avatarId = avatar.GetId()
	local petId = unit.GetActivePet(avatarId)
	local sizeX, sizeY
    if params.source == avatarId or (petId and params.source == petId) then
	  damagevisualization:Show(true)  
	  local panel = Get3DWidget(params.target)
	  local targetIsAvatarTarget = params.target == avatar.GetTarget()
	  local rotateTexture = (params.isMiss and 1.57) or (params.isDodge and -1.57) or 0
	  local textScale = GetTextScale(params.amount)
	  sizeX, sizeY = CalculateSize(textScale, _G["3DWidgetScale"])
	  sizeX = sizeX * _G["3DWidgetWidthMultiplier"]
	  panel:Show(not targetIsAvatarTarget)
	  local textWidget, texturePanel = 
		CreateVisual(panel, GetRandomPosX(_G["3DWidgetWidth"], sizeX), GetRandomPosY(_G["3DWidgetHeight"]),
					 params.amount, GetTextFormatDamage(params.sysSubElement, params.isCritical, params.isGlancing),
					 GetTexture(params), _G["3DWidgetScale"], _G["3DWidgetWidthMultiplier"])
		if texturePanel then
		  texturePanel:Rotate(rotateTexture)
		end
	  if targetIsAvatarTarget then
	    textWidget:Show(false)
		texturePanel:Show(false)
	    textScale = GetTextScale(params.amount)
		sizeX, sizeY = CalculateSize(textScale, TargetWidgetScale)
		local x, y, left, top = targetPanelLayout:CalculatePosition(sizeX, sizeY)
		if x then
	      local targetTextWidget, targetTexturePanel = 
		    CreateVisual(targetpanel, left, top,
			  		     params.amount, GetTextFormatDamage(params.sysSubElement, params.isCritical, params.isGlancing),
					     GetTexture(params), _G["TargetWidgetScale"])
						
		  targetPanelLayout:Register(targetTextWidget, x, y, sizeX, sizeY)
		  targetWidgets[common.RequestIntegerByInstanceId(textWidget:GetInstanceId())] = targetTextWidget
		  targetWidgets[common.RequestIntegerByInstanceId(texturePanel:GetInstanceId())] = targetTexturePanel
		  if targetTexturePanel then
		    targetTexturePanel:Rotate(rotateTexture)
		  end
		end
	  end
	end
  end
--------------------------------------------------------------------------------
--- REACTIONS
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
--- INITIALIZATION
--------------------------------------------------------------------------------
function InitializeTargetPanel()
  damagevisualization = stateMainForm:GetChildChecked("ContextDamageVisualization", false)
  local enemypanel = damagevisualization:GetChildChecked("EnemyPanel", false)
  local children = enemypanel:GetNamedChildren()
  for k, wtChild in children do
	local slidePanel = wtChild:GetChildUnchecked("SlidePanel", false)
	if slidePanel then
	  slidePanel:Show(false)
	end
  end
  targetpanel:SetName("CircleCombatText")
  enemypanel:AddChild(targetpanel)

  targetPanelLayout = CreateTargetPanelLayout()

  -- Lock the middle area 
  targetPanelLayout:Lock(HalfTargetGridColumns - 1, HalfTargetGridColumns - 2, 2, 4)
end
function Init()
  InitializeTargetPanel()
end

CallWhenCharacterAvailable(Init)



	


